//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Session/SessionXML.h
//! @brief     Defines reader and writer classes for SessionModel
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_SESSION_SESSIONXML_H
#define BORNAGAIN_GUI_MODEL_SESSION_SESSIONXML_H

#include "Base/Types/Complex.h"
#include "Base/Vector/Vectors3D.h"
#include <QVariant>

class QXmlStreamWriter;
class QXmlStreamReader;
class SessionItem;
class MessageService;

namespace GUI::Session::XML {

const QString ItemMimeType = "application/org.bornagainproject.xml.item.z";
const QString LinkMimeType = "application/org.bornagainproject.fittinglink";

const QString InstrumentModelTag("InstrumentModel");
const QString SampleModelTag("SampleModel");
const QString MaterialModelTag("MaterialModel");
const QString JobModelTag("JobModel");
const QString DocumentModelTag("DocumentModel");
const QString RealDataModelTag("RealDataModel");

const QString TagAttribute("Tag");

const QString ItemTag("Item");
const QString ModelTypeAttribute("ModelType");
const QString DisplayNameAttribute("DisplayName");
const QString ParameterTag("Parameter");
const QString BinaryData("BinaryData");
const QString NonSessionItemData("NonSessionItemData");
const QString Version("Version");
const QString ParameterNameAttribute("ParName");
const QString ParameterTypeAttribute("ParType");
const QString ParameterValueAttribute("ParValue");
const QString ParameterRoleAttribute("ParRole");
const QString ParameterExtAttribute("ParExt");

const QString ExternalPropertyColorAtt("Color");
const QString ExternalPropertyIdentifierAtt("Identifier");

void writeModel(QXmlStreamWriter* writer, SessionItem* modelRootItem);
void writeItemAndChildItems(QXmlStreamWriter* writer, const SessionItem* item);

//! Write the variant as a complete tag, including the given role
void writeVariant(QXmlStreamWriter* writer, QVariant variant, int role);

//! Write the variant's value as an attribute
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeName,
                    const QVariant& variant);

void writeAttribute(QXmlStreamWriter* writer, const QString& attributeName, int d);
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeName, unsigned d);
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeName, double d);
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeBaseName, const R3& vec);
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeBaseName, const complex_t& c);

void readItems(QXmlStreamReader* reader, SessionItem* parent, QString topTag = "",
               MessageService* messageService = nullptr);
QString readProperty(QXmlStreamReader* reader, SessionItem* item,
                     MessageService* messageService = nullptr);

bool readBoolAttribute(QXmlStreamReader* reader, const QString& attributeName);
unsigned readUIntAttribute(QXmlStreamReader* reader, const QString& attributeName);

void readAttribute(QXmlStreamReader* reader, const QString& attributeName, double* d);
void readAttribute(QXmlStreamReader* reader, const QString& attributeName, QString* s);
void readAttribute(QXmlStreamReader* reader, const QString& attributeName, QColor* c);
void readAttribute(QXmlStreamReader* reader, const QString& attributeBaseName, R3* vec);
void readAttribute(QXmlStreamReader* reader, const QString& attributeBaseName, complex_t* c);

} // namespace GUI::Session::XML

#endif // BORNAGAIN_GUI_MODEL_SESSION_SESSIONXML_H
